@@ -23,7 +23,7 @@
 **********************************************************************
     brgtconv.c: output converters for B-Right/V(TM) variants
 	v1.92	Newly written from ucodeconv.c 
-    $Id: brgtconv.c,v 1.31 2015/03/02 14:21:08 seiji Exp $
+    $Id: brgtconv.c,v 1.36 2018/09/02 07:22:24 seiji Exp seiji $
 **/
 
 #include <stdio.h>
@@ -122,7 +122,8 @@ static void  BRGT_TURNFULL()
 }
 
 /* --------------------------------------------------------------- */
-void SKFBRGT1OUT(x)	/* currently not supported by B-Right/V	   */
+void SKFBRGT1OUT(x)
+			/* currently not supported by B-Right/V	   */
 skf_ucode	x;
 {
     if (brgt_ucod) {
@@ -176,22 +177,12 @@ skf_ucode	x;
     };
 }
 
-void SKFBRGTX0212OUT(x)
+static void SKFBRGTX0212OUT(x)
 skf_ucode	x;
 {
 	SKFBRGTOUT(x);
 }
 
-void SKFBRGTK1OUT(x)
-skf_ucode	x;
-{
-    if (brgt_ucod) {
-	SKFputc(0xfe); SKFputc(0x21);	/* back to the system plane */
-	brgt_ucod = 0;
-    };
-    out_undefined(x,SKF_UX0212);
-}
-
 /* --------------------------------------------------------------- */
 static unsigned short brgt_subdeco0[] = {
   0xff,0xa4,0x00,0x06, 0x04,0x81,0x00,0x00,
@@ -201,8 +192,8 @@ static unsigned short brgt_subdeco1[] = {
   0xff,0xa4,0x00,0x02, 0x05,0x00,0x100
 };
 
-void BRGTSUBSCRIPT(x)	/* subscript decoration			   */
-int x;
+static void BRGTSUBSCRIPT(x)	/* subscript decoration		   */
+unsigned int x;
 {
     SKF_STRPUT(brgt_subdeco0);
     SKFputc((x >> 8) & 0xff);
@@ -221,7 +212,10 @@ skf_ucode c2;
 
     c1 = c2 & 0x7f;
 #ifdef SKFDEBUG
-    if (is_vv_debug) fprintf(stderr," brgt_ascii: %02x",c1);
+    if (is_vv_debug) {
+    	fprintf(stderr," brgt_ascii: %02x",c1);
+	debugcharout((int)c1);
+    };
 #endif
     if (brgt_hwid == 0) {	/* convert to half-width	   */
 	BRGT_TURNHALF();
@@ -458,6 +452,9 @@ void BRGT_compat_oconv (c2)
 	    };
 	};
     } else {
+	if (brgt_hwid == 0) {  /* convert to half-width	   */
+	    BRGT_TURNHALF();
+	};
 	SKFBRGTUOUT(c2);
     };
 #ifndef	USE_UBUF
@@ -478,11 +475,7 @@ void BRGT_private_oconv (c2)
 #endif
 
     if (c2 < 0xe000) {
-	if (c2 < 0xe880) {
-	    lig_x0213_out(c2);
-	} else {
-	    out_undefined(c2,SKF_IOUTUNI);
-	};
+	lig_x0213_out(c2,0);
 	return;
     };
     if (brgt_hwid) {
@@ -514,7 +507,7 @@ const char *st;
 {
     int len;
 
-    if (brgt_hwid == 0) {	/* convert to half-width	   */
+    if (brgt_hwid == 0) {
 	BRGT_TURNHALF();
     };
     for (len=0; (len<30) && (*st != '\0') ; len++,st++) {
