@@ -22,7 +22,7 @@
 **	arising by using whole or a part of this software.
 **********************************************************************
 ** error.c:	error message related routine in skf
-** $Id: error.c,v 1.141 2016/06/14 14:56:32 seiji Exp seiji $
+** $Id: error.c,v 1.149 2020/11/19 14:38:24 seiji Exp seiji $
 **	out_area_out:	error character warning generation
 **	bogus_out:	same as above
 **
@@ -63,17 +63,19 @@
 #define skfstderr	stdout
 #endif
 
+#define ERRF_LIMIT  (size_t)255
+
 /* ident */
-extern char	*rev;
 static const char	*cpyr = 
-"Copyright (c) S.Kaneko, 1993-2015. All rights reserved.\n";
+"Copyright (c) S.Kaneko, 1993-2016. All rights reserved.\n";
 
 /* error message text   */
 static char	*skf_int_err  = 
 	"skf: internal error. please report! - code %d\n";
 static char	*skf_err_header = "skf: ";
-static char	*skf_err_msg;		/* constant character buffer */
+static const char	*skf_err_msg;	/* constant character buffer */
 static char	*skf_err_buf;		/* constant character buffer */
+static char	*nullconststr = "((null))";
 
 static void	display_version_common P_((int));
 
@@ -82,15 +84,16 @@ static int	locale_enable = 0;
 static int	check_format_flaw P_((const char *));
 
 /* --------------------------------------------------------------- */
-/*@-mustfreefresh@*/
-static char *skf_gettext(in_str)
+/*@-mustfreefresh@*/ /*@-statictrans@*//*@-globstate@*/
+static const char *skf_gettext(in_str)
 const char	*in_str;
 {
      char	*msg_buf;
 
+     if (in_str == NULL) in_str = nullconststr;
      msg_buf = gettext(in_str);
      if ((locale_enable <= 0) || (check_format_flaw(msg_buf) < 0)) {
-	return((char *)in_str);
+	return(in_str);
      } else {
 	return(msg_buf);
      };
@@ -102,13 +105,26 @@ const char	*in_str;
 
 /* --------------------------------------------------------------- */
 /*@-formatconst@*/
-void	trademark_warn()
+static void	trademark_warn()
 {
     skf_err_msg = skf_gettext
 ("\nCodeset names may include trademarks and hereby acknowledged.\n");
     fprintf(skfstderr,skf_err_msg);
 }
 
+/* --------------------------------------------------------------- */
+#if SKFDEBUG
+/*@+ignorequals@*/
+static void entry_dump(entry)
+struct iso_byte_defs *entry;
+{
+    fprintf(skfstderr," %s(%lnx)\n",entry->desc,
+		((entry->unitbl == NULL) ? 
+			(long int *)(entry->uniltbl) :
+			(long int *)(entry->unitbl)));
+}
+/*@-ignorequals@*/
+#endif
 /* ---------------------------------------------------------------
     in_undefined(), out_undefined(): error reporting 
       these routines are new and self-explanatory error routine.
@@ -189,6 +205,11 @@ skf_gettext("skf: decoding is unsupported under UTF-16\n");
 skf_gettext("skf: Unicode(TM) Language tag is inconsistent\n");
 	    fprintf(skfstderr,skf_err_msg);
 	    break;
+	case SKF_ARIBERR:
+	    skf_err_msg = 
+skf_gettext("skf: This ARIB B24 feature(%4x) is not supported\n");
+	    fprintf(skfstderr,skf_err_msg,c1);
+	    break;
 	case SKF_UNSUPP:
 	    skf_err_msg = 
 skf_gettext("skf: This code(%08x) feature is not supported\n");
@@ -288,6 +309,7 @@ skf_gettext("skf: encoding and output codeset is inconsistent\n");
 	    skf_err_msg = 
 skf_gettext("skf: this code point(u+%04x) output is prohibited\n");
 	    fprintf(skfstderr,skf_err_msg,c1);
+	    donotout = 1;	/* these points are just discarded   */
 	    break;
 	default:	/* various dirty errors .... */
 	    skf_err_msg = skf_gettext(skf_int_err);
@@ -320,6 +342,7 @@ int	reason;
 const char	*t_desc;
 {
 
+    if (t_desc == NULL) t_desc = nullconststr;
     if (disp_warn || force_disp_warn) {
       switch (reason) {
 	case SKF_PRESETFAIL:
@@ -480,98 +503,92 @@ int mode;
     (void)fputc('\n',skfstderr);
 
 #ifdef SKFDEBUG
-  if (is_v_debug || (mode > 0)) {
+    if (is_v_debug || (mode > 0)) {
 #else
-  if (mode > 0) {
+    if (mode > 0) {
 #endif
-    skf_err_msg = skf_gettext("OPTIONS: ");
-    fprintf(skfstderr,skf_err_msg);
+      skf_err_msg = skf_gettext("OPTIONS: ");
+      fprintf(skfstderr,skf_err_msg);
 #ifdef FAST_GETC
-    fprintf(skfstderr,"FG ");
+      fprintf(skfstderr,"FG ");
 #endif
 #ifdef HAVE_GETENV
-    fprintf(skfstderr,"GE ");
+      fprintf(skfstderr,"ENV ");
 #endif
 #ifdef OLD_NEC_COMPAT
-    fprintf(skfstderr,"98 ");
+      fprintf(skfstderr,"98 ");
 #endif
 #ifdef UNIFY_ASCII_JIS
-    fprintf(skfstderr,"UNFY_A_J ");
+      fprintf(skfstderr,"UFY_A_J ");
 #endif
 #ifdef UTF_MIME_RECOVERY
-    fprintf(skfstderr,"UMR ");
+      fprintf(skfstderr,"MIMEREC ");
 #endif
 #if	defined(ENABLE_NLS) && defined(HAVE_LIBINTL_H)
-    fprintf(skfstderr,"NLS ");
+      fprintf(skfstderr,"NLS ");
 #endif
 #ifdef USE_UBUF
-    fprintf(skfstderr,"UB ");
+      fprintf(skfstderr,"UBUF ");
 #endif
 #ifdef KEIS_DETECT
-    fprintf(skfstderr,"KD ");
+      fprintf(skfstderr,"KD ");
 #endif
 #ifdef OLD_KEIS_DETECT
-    fprintf(skfstderr,"OKD ");
+      fprintf(skfstderr,"OKD ");
 #endif
 #ifdef DYNAMIC_LOADING
-    fprintf(skfstderr,"DY ");
+      fprintf(skfstderr,"DY ");
 #endif
 #ifdef SUPPRESS_FJ_CONVENSION
-    fprintf(skfstderr,"!FJ ");
+      fprintf(skfstderr,"!FJ ");
 #endif
 #ifdef SUPPRESS_UNRECOGNIZED_SEQ
-    fprintf(skfstderr,"SUQ ");
+      fprintf(skfstderr,"SUQ ");
 #endif
 #ifdef _LARGEFILE_SOURCE
-    fprintf(skfstderr,"F64 ");
+      fprintf(skfstderr,"F64 ");
 #endif
 #ifdef SKFDEBUG
 #ifdef CD_DEBUG
-    fprintf(skfstderr,"DBGC ");
+      fprintf(skfstderr,"DBGC ");
 #else
-    fprintf(skfstderr,"DBG ");
+      fprintf(skfstderr,"DBG ");
 #endif
 #endif	/* SKFDEBUG */
 #ifdef NKF_NAMETEST
-    fprintf(skfstderr,"NN ");
+      fprintf(skfstderr,"NN ");
 #endif
 #ifdef ALLOW_TYPOGRAPHY
-    fprintf(skfstderr,"AA");
+      fprintf(skfstderr,"AA");
 #endif
 #ifdef _WIN32
-    fprintf(skfstderr,"WIN ");
+      fprintf(skfstderr,"WIN32 ");
 #endif
 #ifdef GEN_SCRIPT_SUPRT
 #ifdef GEN_SCRIPT_ULM
-    fprintf(skfstderr,"!ULM ");
+      fprintf(skfstderr,"!ULM ");
 #else
-    fprintf(skfstderr,"ULM ");
+      fprintf(skfstderr,"ULM ");
 #endif
 #endif
 #ifdef HAVE_GETUID
 #ifdef HAVE_GETEUID
-    fprintf(skfstderr,"EUID ");
+      fprintf(skfstderr,"EUID ");
 #else
-    fprintf(skfstderr,"UID ");
+      fprintf(skfstderr,"UID ");
 #endif
 #endif
-#ifdef ENABLE_DISABLED
-    fprintf(skfstderr,"ED ");
-#endif
-#ifdef HAVE_MKSTEMP
-    fprintf(skfstderr,"OMST ");
+#ifndef HAVE_MKSTEMP
+      fprintf(skfstderr,"OMST ");
 #endif
 #ifdef SKF_EXPERIMENTAL
-    fprintf(skfstderr,"EXP ");
+      fprintf(skfstderr,"EXP ");
 #endif
-    (void)fputc('\n',skfstderr);
-  } else;
-
-  skf_err_msg = skf_gettext("FEATURES: ");
-  fprintf(skfstderr,skf_err_msg);
-#ifdef KUNIMOTO
+      (void)fputc('\n',skfstderr);
+    } else;
+    skf_err_msg = skf_gettext("FEATURES: ");
+    fprintf(skfstderr,skf_err_msg);
     fprintf(skfstderr,"KN ");
-#endif
 #ifdef UCS2_KANA_SUPPR
     fprintf(skfstderr,"UK ");
 #endif
@@ -595,6 +612,12 @@ int mode;
 #ifdef SUPERFOLD
     fprintf(skfstderr,"SFD ");
 #endif
+#if defined(SWIG_EXT) && defined(SKF_PYTHON3)
+    fprintf(skfstderr,"PY3 ");
+#endif
+#if defined(SWIG_EXT) && defined(SKF_RUBY19)
+    fprintf(skfstderr,"RBC ");
+#endif
 #endif
     if (is_lineend_thru) fprintf(skfstderr,"LE_THRU ");
     if (is_lineend_crlf) fprintf(skfstderr,"LE_CRLF ");
@@ -610,11 +633,11 @@ int mode;
 		(int)((skf_input_lang >> 8) & 0x7fU),
 		(int)(skf_input_lang & 0x7fU));
 	} else fprintf(skfstderr,"lang: neutral ");
-#ifdef DYNAMIC_LOADING
-	skf_err_msg = skf_gettext("Code table dir: %s\n");
-	fprintf(skfstderr,skf_err_msg,skf_ext_table);
-#endif
     } else;
+#endif
+#if defined(DYNAMIC_LOADING) && !defined(SKF_SWIG)
+    skf_err_msg = skf_gettext("Code table dir: %s\n");
+    fprintf(skfstderr,skf_err_msg,skf_ext_table);
 #endif
     if (is_nkf_compat) {
     	fprintf(skfstderr,"NKFOPT: ");
@@ -630,15 +653,17 @@ int mode;
 	fprintf(skfstderr,"STDOUT ");
 #endif
 #ifdef SHIFTJIS_CP932
-	fprintf(skfstderr,"SJIS_IS_CP932 ");
+	fprintf(skfstderr,"CP932EQV ");
 #endif
 	fprintf(skfstderr,"\n");
     } else;
 #ifdef SKFDEBUG
     if (mode > 1) {
     	debug_opt_save = debug_opt;
-	set_vv_debug;
-	debug_analyze();
+	if (is_v_debug) {
+	    set_vv_debug;
+	    debug_analyze();
+	} else;
 	debug_opt = debug_opt_save;
     } else;
 #endif
@@ -670,26 +695,26 @@ void display_nkf_help()
     fprintf(skfstderr," -w                -z\n");               
     fprintf(skfstderr," -W                -Z\n");               
     fprintf(skfstderr," -w16L             --oc=utf-16le-bom\n");
-    fprintf(skfstderr," -W16L             --ic=utf-16le\n");               
-    fprintf(skfstderr," -w32L             --oc=utf-32le-bom\n"); 
-    fprintf(skfstderr," -W32L             --ic=utf-32le\n");               
+    fprintf(skfstderr," -W16L             --ic=utf-16le\n");   
+    fprintf(skfstderr," -w32L             --oc=utf-32le-bom\n");
+    fprintf(skfstderr," -W32L             --ic=utf-32le\n");  
     fprintf(skfstderr," -Z0               --enable-ascii-conv\n");
     fprintf(skfstderr," -Z1               --enable-ascii-conv --enable-space-conv\n");
-    fprintf(skfstderr," -Z2               --enable-ascii-conv --enable-space-sinble-conv\n");
+    fprintf(skfstderr," -Z2               --enable-ascii-conv --enable-space-single-conv\n");
     fprintf(skfstderr," -Z3               --enable-ascii-conv --html-sanitize\n");
     fprintf(skfstderr," -l                --ic=iso-2022-jp-2 --oc=iso-2022-jp-2\n");
     fprintf(skfstderr," -x                --kana-enable\n");
     fprintf(skfstderr," --cap-input       --decode=cap\n");
     fprintf(skfstderr," --url-input       --decode=url\n");
     fprintf(skfstderr," --numchar-input   --decode=hex\n");
-    fprintf(skfstderr," --guess=1         --inquiry\n");
+    fprintf(skfstderr," --guess=1         -g\n");
     fprintf(skfstderr," --guess=2         --hard-inquiry\n");
     fprintf(skfstderr," --oc=utf-8-bom    -z --enable-endian-mark\n");
     fprintf(skfstderr," ----------------- ----------------------------------------------\n");
     skf_err_msg = skf_gettext(
-    	" * following options are not supported: -i -o -T --prefix --fb-skip \n");
+    	" * following options are not supported: -i -o -T --prefix --no-cp932ext\n");
     (void)fputs(skf_err_msg, skfstderr);
-    fprintf(skfstderr,"   --fb-java --no-cp932ext --no-best-fit-chars --ic=utf8-mac\n");
+    fprintf(skfstderr,"   --suffix=* --no-best-fit-chars --ic=utf8-mac\n");
     return;
 }
 /* --------------------------------------------------------------- */
@@ -747,6 +772,11 @@ char *cp;
 "skf: no codeset is specified in command line argument\n");
 	    fprintf(skfstderr,skf_err_msg);
 	    break;
+	case SKF_DEPRECATOPT:
+    skf_err_msg = skf_gettext(
+"skf: this option has been deprecated (%s)\n");
+	    fprintf(skfstderr,skf_err_msg,cp);
+	    break;
 	case SKF_UNDEFOPT:	/*@FALLTHROUGH@*/
 	default:
     skf_err_msg = skf_gettext("skf: unknown option %s\n");
@@ -765,28 +795,26 @@ void initialize_error()
 {
 #if	defined(ENABLE_NLS) && defined(HAVE_LIBINTL_H)
     char *bt,*tl;
+
     if (skf_err_buf == NULL) {
 	if (setlocale(LC_ALL,"") != NULL) {
 	    bt = bindtextdomain(PACKAGE,LOCALEDIR);
 	    tl = textdomain(PACKAGE);
 	    if ((bt == NULL) || (tl == NULL)) {
-		skferr(SKF_MALLOCERR,(long)0,(long)0);
-		skf_exit(EXIT_FAILURE);
+		skferr(SKF_MALLOCERR,(long)32,(long)0);
 	    };
 	    locale_enable = 1;
 	};
-	skf_err_buf = (char *)calloc(256,sizeof(char));
+	skf_err_buf = (char *)calloc((size_t)256,sizeof(char));
 	if (skf_err_buf == NULL) {
-	    skferr(SKF_MALLOCERR,(long)0,(long)0);
-	    skf_exit(EXIT_FAILURE);
+	    skferr(SKF_MALLOCERR,(long)32,(long)1);
 	};
     } else;
 #else
     if (skf_err_buf == NULL) {
-	skf_err_buf = (char *)calloc(256,sizeof(char));
+	skf_err_buf = (char *)calloc((size_t)256,sizeof(char));
 	if (skf_err_buf == NULL) {
-	    skferr(SKF_MALLOCERR,(long)0,(long)0);
-	    skf_exit(EXIT_FAILURE);
+	    skferr(SKF_MALLOCERR,(long)32,(long)2);
 	};
     } else;
 #endif
@@ -795,7 +823,7 @@ void initialize_error()
 /*@+mustfreefresh@*/
 
 /* --------------------------------------------------------------- */
-/* skferr: various serious error				   */
+/* skferr: various serious errors				   */
 /* --------------------------------------------------------------- */
 /*@-globstate@*/
 void skferr(code,a1,a2)
@@ -806,40 +834,19 @@ long	a1,a2;
 
     if (code >= SKF_ERRDUMP) {
 	skf_err_msg = skf_gettext(skf_int_err);
-	fprintf(skfstderr,skf_err_msg, code);
-	fprintf(skfstderr,
+	fprintf(stderr,skf_err_msg, code);
+	fprintf(stderr,
 "dump: (a1: %lx a2: %lx)\n in_code:%d conv_cap:%08lx conv_alt:%08lx\n",
 	    a1,a2,in_codeset,conv_cap,conv_alt_cap);
-	fprintf(skfstderr," g0_table:%08lx",(unsigned long)g0_table_mod);
-	if (g0_table_mod != NULL) 
-	    fprintf(skfstderr," table:%08lx dbyte:%d limit:%d kana:%lx desc:%s",
-		(unsigned long)g0_table_mod->unitbl,
-		g0_table_mod->char_width-1,
-		g0_table_mod->table_len,g0_table_mod->is_kana,
-		g0_table_mod->desc);
-	fprintf(skfstderr,"\n g1_table:%08lx",(unsigned long)g1_table_mod);
-	if (g1_table_mod != NULL) 
-	    fprintf(skfstderr," table:%08lx dbyte:%d limit:%d kana:%lx desc:%s",
-		(unsigned long)g1_table_mod->unitbl,
-		g1_table_mod->char_width-1,
-		g1_table_mod->table_len,g1_table_mod->is_kana,
-		g1_table_mod->desc);
-	fprintf(skfstderr,"\n g2_table:%08lx",(unsigned long)g2_table_mod);
-	if (g2_table_mod != NULL) 
-	    fprintf(skfstderr," table:%08lx dbyte:%d limit:%d kana:%lx desc:%s",
-		(unsigned long)g2_table_mod->unitbl,
-		g2_table_mod->char_width-1,
-		g2_table_mod->table_len,g2_table_mod->is_kana,
-		g2_table_mod->desc);
-	fprintf(skfstderr,"\n g3_table:%08lx",(unsigned long)g3_table_mod);
-	if (g3_table_mod != NULL) 
-	    fprintf(skfstderr," table:%08lx dbyte:%d limit:%d kana:%lx desc:%s",
-		(unsigned long)g3_table_mod->unitbl,
-		g3_table_mod->char_width-1,
-		g3_table_mod->table_len,g3_table_mod->is_kana,
-		g3_table_mod->desc);
-	fprintf(skfstderr,"\n low_table:%08lx\n",(unsigned long)low_table);
-	fprintf(skfstderr," up_table:%08lx\n",(unsigned long)up_table);
+ 	dump_table_address(g0_table_mod,"g0");
+	fprintf(stderr,"\n ");
+ 	dump_table_address(g1_table_mod,"g1");
+	fprintf(stderr,"\n ");
+ 	dump_table_address(g2_table_mod,"g2");
+	fprintf(stderr,"\n ");
+ 	dump_table_address(g3_table_mod,"g3");
+	fprintf(stderr,"\n low_table:%08lx\n",(unsigned long)low_table);
+	fprintf(stderr," up_table:%08lx\n",(unsigned long)up_table);
     } else if (code >= SKF_TABLEERR_G0A) {
       switch(code) {
 	case SKF_TABLEERR_G0A:
@@ -847,78 +854,94 @@ long	a1,a2;
 		"Generic g%1dalt table loading error (table: %s)\n");
 	    tdefs = gx_table_mod; code++; break;
 	case SKF_TABLEERR_G0:
-	    tdefs = g0_table_mod; 	/*@FALLTHROUGH@*/
+	    skf_err_msg = skf_gettext(
+		"Generic g%1d table loading error (table: %s)\n");
+	    tdefs = g0_table_mod; break;
 	case SKF_TABLEERR_G1:
-	    if (tdefs == NULL) tdefs = g1_table_mod; /*@FALLTHROUGH@*/
+	    skf_err_msg = skf_gettext(
+		"Generic g%1d table loading error (table: %s)\n");
+	    tdefs = g1_table_mod; break;
 	case SKF_TABLEERR_G2:
-	    if (tdefs == NULL) tdefs = g2_table_mod; /*@FALLTHROUGH@*/
+	    skf_err_msg = skf_gettext(
+		"Generic g%1d table loading error (table: %s)\n");
+	    tdefs = g2_table_mod; break;
 	case SKF_TABLEERR_G3:
 	    skf_err_msg = skf_gettext(
 		"Generic g%1d table loading error (table: %s)\n");
-	    if (tdefs == NULL) tdefs = g3_table_mod;
-	    break;
+	    tdefs = g3_table_mod; break;
+	case SKF_TABLEERR_O:
+	    skf_err_msg = skf_gettext(
+		"Generic g%1d table loading error (table: %s)\n");
+	    tdefs = g0_table_mod; break;
 	default:
 	    skf_err_msg = skf_gettext(
-	    "unassigned error(%s)\n");
+	    "unassigned error(%d)\n");
+	    fprintf(stderr,skf_err_msg,a1);
       };
       (void)fputs(skf_err_header,skfstderr);  /* header		   */
-      fprintf(skfstderr,skf_err_msg,(code - SKF_TABLEERR_G0),
+      fprintf(stderr,skf_err_msg,(code - SKF_TABLEERR_G0),
 	(tdefs == NULL) ? "(null)" : tdefs->desc);
     } else {
-      (void)fputs(skf_err_header,skfstderr);	/* header	   */
+      (void)fputs(skf_err_header,stderr);	/* header	   */
       switch (code) {
 	case SKF_OBUFREERR:
-	    fprintf(skfstderr,"re-");		/*@FALLTHROUGH@*/
+	    fprintf(stderr,"re-");		/*@FALLTHROUGH@*/
 	case SKF_OBUFERR:	
-	    fprintf(skfstderr,"obuf ");		/*@FALLTHROUGH@*/
+	    fprintf(stderr,"obuf ");		/*@FALLTHROUGH@*/
 	case SKF_MALLOCERR:
-	    skf_err_msg = skf_gettext("failed to allocate buffer(%d)\n");
-	    fprintf(skfstderr,skf_err_msg,code);
+	    skf_err_msg = skf_gettext("failed to allocate buffer(%d-%d)\n");
+	    fprintf(stderr,skf_err_msg,a1,a2);
+	    break;
+	case SKF_IBUFERR:	
+	    fprintf(stderr,"ibuf ");
+	    skf_err_msg = skf_gettext("failed to allocate ibuffer(%d-%d)\n");
+	    fprintf(stderr,skf_err_msg,a1,a2);
 	    break;
 	case SKF_EUCPRESETERR:
 	    skf_err_msg = skf_gettext(
 		"EUC table loading error\n");
-	    (void)fputs(skf_err_msg, skfstderr); break;
+	    (void)fputs(skf_err_msg, stderr); break;
 	case SKF_TBLALLOCERR:
 	    skf_err_msg = skf_gettext(
 		"failed to allocate table for conversion\n");
-	    (void)fputs(skf_err_msg, skfstderr);
+	    (void)fputs(skf_err_msg, stderr);
 	    break;
 	case SKF_DECOMPERR:
 	    skf_err_msg = 
 		skf_gettext("decompose internal sequencer failed\n");
-	    (void)fputs(skf_err_msg, skfstderr);
+	    (void)fputs(skf_err_msg, stderr);
 	    break;
 	case SKF_INTERNALERR:
 	    skf_err_msg = 
 		skf_gettext("decode internal sequencer failed\n");
-	    (void)fputs(skf_err_msg, skfstderr);
+	    (void)fputs(skf_err_msg, stderr);
 	    break;
 	case SKF_PUTFAILERR:
 	    skf_err_msg = 
 		skf_gettext("can't send output character\n");
-	    (void)fputs(skf_err_msg, skfstderr);
+	    (void)fputs(skf_err_msg, stderr);
 	    break;
 	case SKF_DEBUGERR_1:
 	    skf_err_msg = 
 		skf_gettext("skf debug error %d");
-	    fprintf(skfstderr,skf_err_msg, 1);
-	    fprintf(skfstderr,"(%lx,%lx)\n",a1,a2);
+	    fprintf(stderr,skf_err_msg, 1);
+	    fprintf(stderr,"(%lx,%lx)\n",a1,a2);
 	    break;
 	case SKF_DEBUGERR_2:
 	    skf_err_msg = 
 		skf_gettext("skf debug error %d");
-	    fprintf(skfstderr,skf_err_msg, 2);
-	    fprintf(skfstderr,"(%lx,%lx)\n",a1,a2);
+	    fprintf(stderr,skf_err_msg, 2);
+	    fprintf(stderr,"(%lx,%lx)\n",a1,a2);
 	    break;
 	default:
 	    skf_err_msg = skf_gettext("unassigned error(%s)\n");
-	    fprintf(skfstderr,skf_err_msg, "default");
+	    fprintf(stderr,skf_err_msg, "default");
       };
     };
 #ifdef SWIG_EXT
     skf_swig_result = code;
 #endif
+    skf_exit(EXIT_FAILURE);	/* for extra care */
 }
 
 /* --------------------------------------------------------------- */
@@ -929,32 +952,47 @@ int	mode;
     int res;
 
     (void)fflush(stdout);
+    if (fnam == NULL) fnam = nullconststr;
     if (mode == 1) {
-	res = snprintf(skf_err_buf,255,
+	res = snprintf(skf_err_buf,ERRF_LIMIT,
 		skf_gettext("skf: can't open output file %s\n"), fnam);
     } else if (mode == 2) {
-	res = snprintf(skf_err_buf,255,
+	res = snprintf(skf_err_buf,ERRF_LIMIT,
 		skf_gettext("skf: can't stat input file %s\n"), fnam);
     } else if (mode == 3) {
-	res = snprintf(skf_err_buf,255,
+	res = snprintf(skf_err_buf,ERRF_LIMIT,
 		skf_gettext("skf: can't chmod output file %s\n"), fnam);
-    } else if (mode == 4) {
-	res = snprintf(skf_err_buf,255,
+    } else if (mode == 4) {	/* this case is not quite serious */
+	res = snprintf(skf_err_buf,ERRF_LIMIT,
 		skf_gettext("skf: can't set date output file %s\n"), fnam);
+	return;
     } else if (mode == 5) {
-	res = snprintf(skf_err_buf,255,
+	res = snprintf(skf_err_buf,ERRF_LIMIT,
 		skf_gettext("skf: can't remove input file %s\n"), fnam);
     } else if (mode == 6) {
-	res = snprintf(skf_err_buf,255,
+	res = snprintf(skf_err_buf,ERRF_LIMIT,
 		skf_gettext("skf: can't rename output file %s\n"), fnam);
+#ifdef SWIG_EXT
+    } else if (mode == 7) {	/* this case is not serious	  */
+        if (disp_warn || force_disp_warn) {
+	    res = snprintf(skf_err_buf,255,
+		skf_gettext("skf: can't open input string\n"));
+	    (void)fflush(skfstderr);
+	} else;
+	skf_swig_result = SKF_OPENERR;
+	return;
+#endif
     } else if (mode == 8) {
-	res = snprintf(skf_err_buf,255,
+	res = snprintf(skf_err_buf,ERRF_LIMIT,
 		skf_gettext("skf: in-place/overwrite are unsupported on this plathome\n"));
     } else if (mode == 9) {
-	res = snprintf(skf_err_buf,255,
+	res = snprintf(skf_err_buf,ERRF_LIMIT,
 		skf_gettext("skf: temp-file name generation failed\n"));
+    } else if (mode == 10) {
+	res = snprintf(skf_err_buf,ERRF_LIMIT,
+		skf_gettext("skf: this suffix string is prohibited\n"));
     } else {
-	res = snprintf(skf_err_buf,255,
+	res = snprintf(skf_err_buf,ERRF_LIMIT,
 		skf_gettext("skf: can't open input file %s\n"), fnam);
     };
     if ((res >= 0) && (res <= 256)) {	/* glibc workaround	   */
@@ -1056,18 +1094,18 @@ void	skf_incode_display()
 void	skf_outcode_display()
 {
 /* name and language of the codeset */
-    if ((out_codeset > 0) && (out_codeset < codeset_end)) {
+    if ((out_codeset < 0) || (out_codeset >= codeset_end)) {
     	out_codeset = DEFAULT_O;
     } else;
     fprintf(skfstderr,"%s (#%d,%x%x,typ:%lx) ",
-	    i_codeset[out_codeset].desc,out_codeset,
+	i_codeset[out_codeset].desc,out_codeset,
 	    ((i_codeset[out_codeset].oconv_lang) >> 8) & 0x7f,
 	    (i_codeset[out_codeset].oconv_lang) & 0x7f,
 	    i_codeset[out_codeset].oconv_type);
     (void)fflush(skfstderr);
 }
 /* --------------------------------------------------------------- */
-/*@-branchstate@*/
+/*@-branchstate@*//*@+matchanyintegral@*/
 void test_support_charset()
 {
     int i,j;
@@ -1082,7 +1120,9 @@ void test_support_charset()
 
     (void)fflush(skfstderr); (void)fflush(stdout);
     for (i=0; iso_ubytedef_table[i].ientry != NULL; i++) { 
-    	if ((i == ub_ovlay_index) || (i == prv_ovlay_index)) continue;
+    	/* do not display these stuffs */
+        if ((i == ub_ovlay_index) || (i == ub_prv_drcs_index) ||
+	    (i == ub_prv_drcsdbl_index)) continue;
 	fprintf(skfstderr,"# %s:\n",iso_ubytedef_table[i].desc);
 	uentry = iso_ubytedef_table[i].ientry;
 	for (j=0; uentry[j].defschar != 0; j++) {
@@ -1096,10 +1136,7 @@ void test_support_charset()
 	      if (uentry[j].desc != NULL) {
 #if SKFDEBUG
 		if (is_v_debug) {
-		    fprintf(skfstderr," %s(%08lx)\n",uentry[j].desc,
-		    ((uentry[j].unitbl == NULL) ? 
-			(unsigned long)uentry[j].uniltbl :
-			(unsigned long)uentry[j].unitbl));
+		    entry_dump(&uentry[j]);
 		} else;
 #endif
 		fprintf(skfstderr,"%s%s%s\n",
@@ -1119,7 +1156,7 @@ void test_support_charset()
     };
     fprintf(skfstderr,"# Unicode(TM)\n");
     fprintf(skfstderr," -\t\tUTF-16/UCS2\n -\t\tUTF-8\n -\t\tUTF-7\n");
-    fprintf(skfstderr," -\t\tCESU-8\n");
+    fprintf(skfstderr," -\t\tCESU-8\n -\t\tUTF7-IMAP(RFC3501)\n");
 	trademark_warn();
 }
 /* --------------------------------------------------------------- */
@@ -1146,13 +1183,13 @@ void test_support_codeset()
     };
 	trademark_warn();
 }
-/*@+branchstate@*/
+/*@+branchstate@*//*@-matchanyintegral@*/
 /* --------------------------------------------------------------- */
 #ifdef SKFDEBUG
 /*@-mustfreefresh@*/
 void debug_analyze()
 {
-    char *trans_txt;
+    const char *trans_txt;
 
     if (is_v_debug) {
 	trademark_warn();
@@ -1167,7 +1204,6 @@ void debug_analyze()
 	if (is_lineend_crlf) fprintf(skfstderr,"LE_CRLF ");
 	if (is_lineend_cr) fprintf(skfstderr,"LE_CR ");
 	if (is_lineend_lf) fprintf(skfstderr,"LE_LF ");
-	if (use_latin2x0212) fprintf(skfstderr,"x0212_latin ");
 	if (use_latin2tex) fprintf(skfstderr,"tex_latin ");
 	if (use_latin2htmlu) fprintf(skfstderr,"uri_latin ");
 	if (use_latin2htmlh) fprintf(skfstderr,"uri_latin(hex) ");
@@ -1179,6 +1215,7 @@ void debug_analyze()
 	if (use_ms_compat) fprintf(skfstderr,"ms_compat ");
 	if (o_add_bom) fprintf(skfstderr,"add_bom ");
 	if (limit_ucs2) fprintf(skfstderr,"limit_ucs2 ");
+	if (sup_jis90) fprintf(skfstderr,"dsbl_jis90 ");
 	if (is_ucs_ufam(conv_cap)) {
 	    if (out_endian(conv_cap)) {
 	    	fprintf(skfstderr,"BE ");
@@ -1256,9 +1293,7 @@ void debug_analyze()
 	if (is_ucs_utf8(conv_cap)) 
 		fprintf(skfstderr,"UTF-8 little endian output ");
 	if (is_nkf_compat) fprintf(skfstderr,"nkf_CMPT ");
-#ifdef KUNIMOTO
 	if (kuni_opt) fprintf(skfstderr,"LW_DET");
-#endif
 	if (is_nkf_jbroken) fprintf(skfstderr,"JBRKN ");
 	if (is_nkf_jfbroken) fprintf(skfstderr,"DBRKN ");
 	if (is_nkf_jfbroken) fprintf(skfstderr,"LBRKN ");
@@ -1287,6 +1322,7 @@ void ValidValueDisplay(plane,dispstr)
 int plane;
 char *dispstr;
 {
+    if (dispstr == NULL) dispstr = nullconststr;
     if (is_euc(i_codeset[in_codeset].encode) && (plane == 1)) {
 	skf_err_msg = skf_gettext("skf: g1 is overwritten in EUC\n");
 	fprintf(skfstderr,skf_err_msg);
@@ -1301,6 +1337,7 @@ char *dispstr;
 void ValidLangDisplay(dispstr)
 char *dispstr;
 {
+    if (dispstr == NULL) dispstr = nullconststr;
     skf_err_msg = 
 	skf_gettext("skf: possible language set for skf: %s\n");
     fprintf(skfstderr,skf_err_msg,dispstr);
