@@ -23,7 +23,7 @@
 **********************************************************************
     genoconv.c:	input/output converters seed
 	v1.90	Newly written from unic*de.c parsing
-    $Id: genoconv.c,v 1.142 2015/04/26 15:08:38 seiji Exp seiji $
+    $Id: genoconv.c,v 1.146 2018/12/17 13:56:25 seiji Exp seiji $
 **/
 
 #include <stdio.h>
@@ -100,8 +100,11 @@ void @gen@_ascii_oconv (c1)
 /* memo: this routine output extended EUC code when necessary	   */
     c3 = uni_o_ascii[c1];
 #ifdef SKFDEBUG
-    if (is_vv_debug) fprintf(stderr," @gen@_ascii:%02x,%02x(%02x)",
-	((c1 >> 8) & 0xff),(c1 & 0xff),c3);
+    if (is_vv_debug) {
+    	fprintf(stderr," @gen@_ascii:%02x,%02x(%02x)",
+	    ((c1 >> 8) & 0xff),(c1 & 0xff),c3);
+	debugcharout((int)c3);
+    } else;
 #endif
 #if	defined(SKFKEISSW)
     if (c3 >= 0x100) {
@@ -166,12 +169,9 @@ void @gen@_ascii_oconv (c1)
 	};
 #if	defined(SKFJISSW)
     } else if (is_2ln(c3)) {
-    	if (is_3pl(c3)) {
+	if (is_3pl(c3)) {
 	    SKF@gen@1OUT(c3);
-	} else SKF@gen@8859OUT(c3);
-	return;
-    } else if (is_4pl(c3)) {
-    	SKF@gen@8859OUT(c3);
+	} else SKFJIS8859OUT(c3);
 	return;
 #endif
 #if	defined(SKFBGSW)
@@ -274,73 +274,62 @@ void @gen@_latin_oconv (c4)
 		converted = 1;
 	    };
 #if defined(SKFJISSW) 
-	} else if (is_2ln(c3)) {  /* always some kind of iso8859    */
+	} else if (is_2ln(c3)) {  /* some kind of iso8859    */
 	    if (is_3pl(c3)) {
-		SKF@gen@1OUT(c3);
-	    } else SKF@gen@8859OUT(c3);
+	    	SKF@gen@1OUT(c3);
+	    } else SKFJIS8859OUT(c3);
 	    converted = 1;
 #endif
 #if !defined(SKFSJISSW)
 	} else if (is_2pl(c3) &&
 		(use_x0212 || (!is_jis(conv_cap) && !is_euc(conv_cap)))) {
+#if defined(SKFEUCSW)
+	    if (!is_euc_51932(conv_cap) && !is_euc_20932(conv_cap)) {
 	    SKF@gen@G3OUT(c3);
 	    converted = 1;
+	    } else;
+#else /* JIS */
+	    SKF@gen@G3OUT(c3);
+	    converted = 1;
+#endif	/* !SKFSJISSW && !SKFEUCSW */
 	} else if (is_3pl(c3)) {
 	    SKF@gen@G4OUT(c3);
 	    converted = 1;
-#else
+#else	/* SJIS */
 	} else if (is_2pl(c3) && 
 			(is_ms_213c(conv_cap) || is_ms_cel(conv_cap))) {
-#ifdef SKFDEBUG
-	    if (is_vv_debug) {
-		fprintf(stderr," 2P");
-	    } else;
-#endif
 	    SKF@gen@G3OUT(c3);
 	    converted = 1;
 #endif	/* SKFSJISSW */
 	} else;
 #endif
     };
-    if (converted == 0) {
 #if defined(SKFJISSW) 
+    if (converted == 0) {
 	if ((c4 < 0x100) && use_iso8859) { /* use only in iso8859-1 */
 	    SKF@gen@8859OUT(c4);
 	    converted = 1;
-	} else
-#endif
-	if (use_latin2html) {
-		converted = latin2html(c4);
-	} else if (use_latin2tex) {
-		converted = latin2tex(c4);
-	};
+	} else;
     };
-    if (converted == 0) {
-#ifdef SKFSJISSW
-	if (out_codeset == codeset_cp932w) {
-	    out_undefined(c4,SKF_OUNDEF);
-	} else if (c4 < 0x100) {
-#else
-	if (c4 < 0x100) {
 #endif
-	    ascii_fract_conv(c1);
 #if defined (SKFKEISSW)
-	} else if (is_keis_jef(conv_cap) && ((c1 == 0xd2) || (c1 == 0xd4))) {
+    if (converted == 0) {
+	if (is_keis_jef(conv_cap) && ((c1 == 0xd2) || (c1 == 0xd4)) &&
+		(c4 >= 0x100)) {
 	    if (c1 == 0xd2) SKFKEISEOUT(0x7fda);
 	    else SKFKEISEOUT(0x7fdb);
 	    converted = 1;
+	} else;
+    } else;
 #endif
-	} else {
-	    GRPH_lig_conv(c4);  
-	};
-    };
+
+    if (converted == 0) skf_lastresort(c4);
 }
 
 /* --- CJK kana plane -------------------------------------------- */
 /*  handles 0x3000 - 0x4dff					   */
 /* --------------------------------------------------------------- */
 /*@-globstate@*/ /* should we have to check uni_o_kana is not NULL? */
-/*@-nullpass@*/ /* cjk_out_parse gocha@! Should fix later */
 void @gen@_cjkkana_oconv (c2)
     skf_ucode    c2;
 {
@@ -415,7 +404,7 @@ void @gen@_cjkkana_oconv (c2)
 	    } else if (is_2ln(ch)) {
 		if (is_3pl(ch)) {
 		    SKF@gen@1OUT(ch);
-		} else SKF@gen@8859OUT(ch);
+		} else SKFJIS8859OUT(ch);
 		return;
 #endif
 #if !defined(SKFSJISSW)
@@ -432,9 +421,16 @@ void @gen@_cjkkana_oconv (c2)
 		    fprintf(stderr,"2P");
 		} else;
 #endif
+#if	defined(SKFEUCSW)
+		if (!is_euc_51932(conv_cap) && !is_euc_20932(conv_cap)) {
 		SKF@gen@G3OUT(ch);
 		return;
 	    } else;
+#else
+		SKF@gen@G3OUT(ch);
+		return;
+#endif
+	    } else;
 #endif
 	} else;
     };
@@ -475,7 +471,7 @@ void @gen@_cjk_oconv (c3)
 	    SKF@gen@1OUT(c4); 
 	    return;
 	};
-#else
+#else	/* SJIS, JIS, EUC */
 	if (c4 >= 0x100) {
 	    if (c4 < 0x8000) {
 #if defined(SKFEUCSW) || defined(SKFJISSW)
@@ -485,7 +481,15 @@ void @gen@_cjk_oconv (c3)
 #endif
 		return;
 	    } else if (is_2pl(c4) && use_x0212) {
-		SKF@gen@G3OUT(c4); return;
+#if	defined(SKFEUCSW)
+		if (!is_euc_51932(conv_cap) && !is_euc_20932(conv_cap)) {
+		    SKF@gen@G3OUT(c4);
+		    return;
+		} else;
+#else
+		SKF@gen@G3OUT(c4);
+		return;
+#endif
 #if !defined(SKFSJISSW) && !defined(SKFKEISSW)
 	    } else if (is_3pl(c4)) {
 		SKF@gen@G4OUT(c4); return;
@@ -493,10 +497,10 @@ void @gen@_cjk_oconv (c3)
 	    } else;
 #if	defined(SKFJISSW)
 	} else if (is_2ln(c4)) {
-	    if (is_3pl(c4)) {
-		SKF@gen@1OUT(c4);
-	    } else SKF@gen@8859OUT(c4);
-	    return;
+		if (is_3pl(c4)) {
+		    SKF@gen@1OUT(c4);
+		} else SKFJIS8859OUT(c4);
+		return;
 #endif
 	} else if ((c4 != 0) && (c4 < 0x80)) {
 	    SKF@gen@1OUT(c4); return;
@@ -505,7 +509,7 @@ void @gen@_cjk_oconv (c3)
 	    SKF@gen@G2OUT(c4); return;
 #endif
 	};
-#endif
+#endif	/* SJIS, JIS, EUC */
     };
 
     if (converted == 0) skf_lastresort(c3);
@@ -531,12 +535,15 @@ void @gen@_ozone_oconv(c2)
     } else;
 
 #ifdef SKFDEBUG
-    c3 = (c2 >> 8) & 0xfff;
-    c1 = c2 & 0xff;
-    if (is_vv_debug) fprintf(stderr," @gen@_ozone:%03x,%02x",c3,c1);
+    if (is_vv_debug) {
+	c3 = (c2 >> 8) & 0xffffff;
+	c1 = c2 & 0xff;
+    	fprintf(stderr," @gen@_ozone:%03x,%02x",c3,c1);
+    } else;
 #endif
 
     if ((c2 >= HNGL_TBL_START) && (c2 <= HNGL_TBL_END)) {/* Hangles */
+    	/* TODO: this part assumes only rfc1554 has mixed CJK's.  */
 	if (uni_o_hngl != NULL) {
 	    ch = uni_o_hngl[c2 - HNGL_TBL_START];
 #if defined(SKFJISSW)
@@ -602,6 +609,9 @@ void @gen@_ozone_oconv(c2)
 	if ((ch == 0) && (c2 >= 0x1f100) && (c2 < 0x1f200)) {
 	    enc_alpha_supl_conv(c2);
 	    return;
+	} else if ((ch == 0) && (c2 >= 0x1f200) && (c2 < 0x1f300)) {
+	    enc_cjk_supl_conv(c2);
+	    return;
 	} else;
     } else if ((c2 >= CJK_B_TBL_START) && (c2 <= CJK_B_TBL_END)) {
     				/* CJK Ext.B/C/D   */
@@ -617,10 +627,8 @@ void @gen@_ozone_oconv(c2)
     	return;			/* discard TAGS	*/
     } else if ((c2 >= VARSEL_SUP_TBL_START) && (c2 < VARSEL_SUP_TBL_END)) {
     	return;			/* discard VARIATION SELECTORS	*/
-    } else {
-	out_undefined(c2,SKF_NOTABLE);
-	return;
-    };
+    } else;
+
 #ifdef SKFDEBUG
     if (is_vv_debug) {
     	fprintf(stderr," ch(%x)",ch);
@@ -654,7 +662,7 @@ void @gen@_ozone_oconv(c2)
 	} else if (is_2ln(ch)) {
 	    if (is_3pl(ch)) {
 		SKF@gen@1OUT(ch);
-	    } else SKF@gen@8859OUT(ch);
+	    } else SKFJIS8859OUT(ch);
 	    return;
 #endif
 #if !defined(SKFSJISSW)
@@ -694,10 +702,7 @@ void @gen@_ozone_oconv(c2)
 	    ;
 	};
 #endif
-    } else {
-	out_undefined(c2,SKF_OUNDEF);
-	return;
-    };
+    } else;
 
     if (converted == 0) {
 #ifndef SKF_CODECOUNT
@@ -767,14 +772,27 @@ void @gen@_compat_oconv (c2)
 		if (is_2ln(c3)) {
 		    if (is_3pl(c3)) {
 			SKF@gen@1OUT(c3);
-		    } else SKF@gen@8859OUT(c3);
+		    } else SKFJIS8859OUT(c3);
 		    return;
 		} else 
 #endif
 #if !defined(SKFSJISSW)
 	      	if (is_2pl(c3) && use_x0212) {
+#ifdef SKFDEBUG
+		    if (is_vv_debug) {
+			fprintf(stderr,"2P");
+		    } else;
+#endif
+#if	defined(SKFEUCSW)
+		    if (!is_euc_51932(conv_cap) &&
+		    	!is_euc_20932(conv_cap)) {
 		    SKF@gen@G3OUT(c3);
-		    converted = 1;
+			return;
+		    } else;
+#else
+		    SKF@gen@G3OUT(c3);
+		    return;
+#endif
 		} else if (is_3pl(c3)) {
 		    SKF@gen@G4OUT(c3);
 		    converted = 1;
@@ -824,7 +842,7 @@ void @gen@_compat_oconv (c2)
 /* --- private use plane ----------------------------------------- */
 /* --------------------------------------------------------------- */
 #if defined(SKFSJISSW)
-unsigned short celn_prv_map[22] = {
+static unsigned short celn_prv_map[22] = {
     0xf040,0xf985,0xf990,0xf987, 0xf988,0xf989,0xf98a,0xf98b,
     0xf98c,0xf98d,0xf98e,0xf98f, 0x0000,0x0000,0x0000,0x0000,
     0x0000,0x0000,0x0000,0x0000, 0x0000,0x0000
@@ -840,20 +858,6 @@ static unsigned short cels_prv_map[22] = {
     0xfbb0,0xfbaf,0xfbab,0xfbb4, 0xfbb2,0xfbac
 };
 #endif
-#if defined(SKFSJISSW) || defined(SKFEUCSW) || defined(SKFJISSW)
-static unsigned short x213_rev_conv[80] = { /* d800 - d84f */
-    0x0000,0x2477,0x2478,0x2479, 0x247a,0x247b,0x247c,0x2678,
-    0x2577,0x2578,0x2579,0x257a, 0x257b,0x257c,0x257d,0x257e,
-    0x247d,0x247e,0x287d,0x287e, 0x2b44,0x285f,0x2860,0x2861,
-    0x2b48,0x2b49,0x2b4a,0x2b4b, 0x2b4c,0x2b4d,0x2b4e,0x2b4f,
-    0x2b65,0x2b66,0x2862,0x2863, 0x2864,0x2865,0x2866,0x0000,
-    0x0000,0x0000,0x0000,0x0000, 0x2c74,0x2c75,0x2c76,0x2c77,
-    0x2c78,0x2c79,0x2c7a,0x2c7b, 0x2c7c,0x2d58,0x2d59,0x2d5a,
-    0x2d5b,0x2d5c,0x2d5d,0x2d5e, 0x2d70,0x2d71,0x2d72,0x2d74,
-    0x2d75,0x2d76,0x2d77,0x2d7a, 0x2d7b,0x2d7c,0x2e21,0x2f7e,
-    0x4f54,0x4f7e,0x7427,0x7e7a, 0x7e7b,0x7e7c,0x7e7d,0x7e7e
-};
-#endif
 
 void @gen@_private_oconv (c2)	/* u+d800 to u+f8ff		   */
     skf_ucode    c2;
@@ -918,7 +922,7 @@ void @gen@_private_oconv (c2)	/* u+d800 to u+f8ff		   */
 	} else;
 #endif
 	/* others will be processed using ligature	  */
-	lig_x0213_out(c2);
+	lig_x0213_out(c2,0);
 	return;
     } else if (uni_o_prv != NULL) {	/* 0xe000 - 0xf7ff */
 	ch = uni_o_prv[c2 - 0xe000];
@@ -957,12 +961,13 @@ void @gen@_private_oconv (c2)	/* u+d800 to u+f8ff		   */
 			    SKFputc(c1); SKFputc(ch & 0xff);
 			    return;
 			};
-		    };
+		    } else;
 		};
 	    };
 	} else;
 #endif
 #if defined(SKFJISSW)
+#ifdef VODAFONE_GRAPHIC
 	if (is_ms_cel(conv_cap)) { /* vodaf*ne graphics */
 	    if (((c2 >= 0xe001) && (c2 <= 0xe05a))
 	       || ((c2 >= 0xe101) && (c2 <= 0xe15a))
@@ -984,6 +989,9 @@ void @gen@_private_oconv (c2)	/* u+d800 to u+f8ff		   */
 		return;
 	    } else;
 	} else if (is_cp5022x(conv_cap)
+#else
+	if (is_cp5022x(conv_cap)
+#endif
 	    && (c2 < MS_PRIV_LIM)) {
 	    c1 = ((c2 - 0xe000) / 94) + 0x7f;
 	    ch = ((c2 - 0xe000) % 94) + 0x21;
@@ -1033,7 +1041,7 @@ void @gen@_private_oconv (c2)	/* u+d800 to u+f8ff		   */
 /* --- finish process -------------------------------------------- */
 void @gen@_finish_procedure()
 {
-    oconv_flush();
+    oconv_flush();	/* wipe pending charcters to out-streams   */
 #ifdef SKFJISSW
     if (out_jis && si_enbl && is_x0201_lshift) {
 	SKFputc(A_SI); 		/* si locking shift case */
